/**
 * SPDX-FileCopyrightText: Copyright 2023 Open Mobile Platform LLC <community@omp.ru>
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include <embedder_texture/camera_egl_helper.h>
#include <flutter/platform-methods.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

static PFNEGLCREATEIMAGEKHRPROC eglCreateImageKHR;
static PFNEGLDESTROYIMAGEKHRPROC eglDestroyImageKHR;

void CameraEGLHelper::EGLInit()
{
    eglCreateImageKHR = reinterpret_cast<PFNEGLCREATEIMAGEKHRPROC>(
        eglGetProcAddress("eglCreateImageKHR"));
    eglDestroyImageKHR = reinterpret_cast<PFNEGLDESTROYIMAGEKHRPROC>(
        eglGetProcAddress("eglDestroyImageKHR"));
}

EGLImageKHR CameraEGLHelper::EGLCreateImage(
    std::shared_ptr<Aurora::StreamCamera::GraphicBuffer> buffer)
{
    auto display = PlatformMethods::GetEGLDisplay();
    auto context = PlatformMethods::GetEGLContext();

    const void *handle = buffer->handle;
    GLint eglImgAttrs[] = {EGL_IMAGE_PRESERVED_KHR, EGL_TRUE, EGL_NONE, EGL_NONE};
    return eglCreateImageKHR(display,
                             context,
                             EGL_NATIVE_BUFFER_ANDROID,
                             (EGLClientBuffer) handle,
                             eglImgAttrs);
}

void CameraEGLHelper::EGLDestroyImage(EGLImageKHR image)
{
    auto display = PlatformMethods::GetEGLDisplay();

    eglDestroyImageKHR(display, image);
}
